//************************************************************************//
//									  //
//  Copyright 2013 Bertram Kopf (bertram@ep1.rub.de)			  //
//  	      	   Julian Pychy (julian@ep1.rub.de)			  //
//          	   - Ruhr-Universität Bochum 				  //
//									  //
//  This file is part of Pawian.					  //
//									  //
//  Pawian is free software: you can redistribute it and/or modify	  //
//  it under the terms of the GNU General Public License as published by  //
//  the Free Software Foundation, either version 3 of the License, or 	  //
//  (at your option) any later version.	 	      	  	   	  //
//									  //
//  Pawian is distributed in the hope that it will be useful,		  //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of	  //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	  //
//  GNU General Public License for more details.	      		  //
//									  //
//  You should have received a copy of the GNU General Public License     //
//  along with Pawian.  If not, see <http://www.gnu.org/licenses/>.	  //
//									  //
//************************************************************************//

// pbarpTensorLh class definition file. -*- C++ -*-
// Copyright 2012 Bertram Kopf

#pragma once

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <complex>
#include <memory>
#include <boost/function.hpp>
#include <memory>

#include "qft++/topincludes/relativistic-quantum-mechanics.hh"

#include "pbarpUtils/pbarpBaseLh.hh"
#include "PwaUtils/DataUtils.hh"
#include "PwaUtils/AbsChannelEnv.hh"

class AbsXdecAmp;
class pbarpReaction;
class LSDecAmps;
class AbsPawianParameters;

class pbarpTensorLh : public pbarpBaseLh {

public:
  // pbarpTensorLh(std::shared_ptr<const EvtDataBaseList>);
  pbarpTensorLh(ChannelID channelID);

  virtual ~pbarpTensorLh();

  virtual AbsLh* clone_() const {
    AbsLh* theClone=new pbarpTensorLh(_channelID);
    theClone->setDataVec(_evtDataVec);
    theClone->setMcVec(_evtMCVec);
    return theClone;
  }

  virtual double calcEvtIntensity( EvtData* theData, std::shared_ptr<AbsPawianParameters> fitPar);
  virtual void print(std::ostream& os) const;


protected:

  std::map< std::shared_ptr<const JPCLS>, complex<double>, pawian::Collection::SharedPtrLess > _pbarpAmpSingletMap;
  std::map< std::shared_ptr<const JPCLS>, complex<double>, pawian::Collection::SharedPtrLess > _pbarpAmpTriplet0Map;
  std::map< std::shared_ptr<const JPCLS>, complex<double>, pawian::Collection::SharedPtrLess > _pbarpAmpTripletp1Map;
  std::map< std::shared_ptr<const JPCLS>, complex<double>, pawian::Collection::SharedPtrLess > _pbarpAmpTripletm1Map;

private:
  void initialize();

};
